package repo

import (
	"context"
	"database/sql"
	"encoding/json"

	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/adapter/repo/sqlc"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/model"
	"git.vinchent.xyz/vinchent/howmuch/internal/howmuch/usecase/repo"
	"git.vinchent.xyz/vinchent/howmuch/internal/pkg/log"
)

type eventRepository struct {
	db *sql.DB
}

func NewEventRepository(db *sql.DB) repo.EventRepository {
	return &eventRepository{
		db: db,
	}
}

// Create implements repo.EventRepository.
func (e *eventRepository) Create(
	ctx context.Context,
	evEntity *model.EventEntity,
) (*model.EventEntity, error) {
	panic("unimplemented")
}

// Delete implements repo.EventRepository.
func (e *eventRepository) Delete() {
	panic("unimplemented")
}

// GetByID implements repo.EventRepository.
func (e *eventRepository) GetByID(ctx context.Context, eventID int) (*model.EventRetrieved, error) {
	queries := sqlc.New(e.db)
	eventDTO, err := queries.GetEventByID(ctx, int32(eventID))
	if err != nil {
		log.ErrorLog("query error", "err", err)
		return nil, err
	}

	// marshal owner and users
	var owner *model.UserBaseRetrieved
	err = json.Unmarshal(eventDTO.Owner, owner)
	if err != nil {
		// Unexpected
		log.ErrorLog("json unmarshal error", "err", err)
		return nil, err
	}

	var users []*model.UserBaseRetrieved
	err = json.Unmarshal(eventDTO.Owner, &users)
	if err != nil {
		// Unexpected
		log.ErrorLog("json unmarshal error", "err", err)
		return nil, err
	}

	eventRetrieved := &model.EventRetrieved{
		ID:          int(eventDTO.ID),
		Name:        eventDTO.Name,
		Description: eventDTO.Description.String,
		TotalAmount: model.MakeMoney(
			int(eventDTO.TotalAmount.Int32),
			model.Currency(eventDTO.DefaultCurrency),
		),
		DefaultCurrency: model.Currency(eventDTO.DefaultCurrency),
		CreatedAt:       eventDTO.CreatedAt,
		UpdatedAt:       eventDTO.UpdatedAt,
		Owner:           owner,
		Users:           users,
	}

	return eventRetrieved, nil
}

// ListEventsByUserID implements repo.EventRepository.
func (e *eventRepository) ListEventsByUserID(
	ctx context.Context,
	userID int,
) ([]model.EventBaseItemEntity, error) {
	panic("unimplemented")
}

// ListExpensesByUserID implements repo.EventRepository.
func (e *eventRepository) ListExpensesByUserID() {
	panic("unimplemented")
}

// UpdateInfo implements repo.EventRepository.
func (e *eventRepository) UpdateInfo() {
	panic("unimplemented")
}
